using System;
using System.Xml;
using System.Web;
using System.Web.Security;

namespace sstchur.web.authentication
{
	public enum AuthenticationResult { Succeeded, Failed };
	
	public class AuthenticationControl
	{      
		private string m_strUserId;
		private string m_strHashedPassword;
		private string m_strAuthenticationFile;
		private string[] m_arrRoles;
		private string m_strCookieName = "UPSLTAuth:Cookie";
		private string m_strCookieContent = null;
	  
		public string AuthenticationFile
		{
			set { m_strAuthenticationFile = value; }
		} 
	  
		public string[] Roles
		{
			set { m_arrRoles = value; }
		}
		
		public string CookieName
		{
			set { m_strCookieName = value; }
		}
		
		public string CookieContent
		{
			set { m_strCookieContent = value; }
		}
	  
		public AuthenticationResult Authenticate(string _strUserId, string _strPassword)
		{
			m_strUserId = _strUserId;
			m_strHashedPassword = FormsAuthentication.HashPasswordForStoringInConfigFile(_strPassword, "MD5");

			string strXQuery = "User[UserId = \"" + m_strUserId + "\"]";
			
			// Check if the user has an account in the AuthenticationFile
			XmlNode nodeUserInfo = null;
			string strDefaultRole = null;
			if (UserHasValidAccount(strXQuery, ref nodeUserInfo, ref strDefaultRole))
			{
				string strRoles = strDefaultRole + "|";
				if (m_arrRoles != null)
					strRoles += DetermineRoles(m_strUserId);

				CreateAuthenticationTicket(strRoles);
				CreateUserInfoCookie(nodeUserInfo);
				return AuthenticationResult.Succeeded;
			}
			else
				return AuthenticationResult.Failed;        
		}
	  
		private bool UserHasValidAccount(string _strXQuery, ref XmlNode _nodeUserInfo, ref string _strDefaultRole)
		{        
			string strXmlFilePath = HttpContext.Current.Server.MapPath(m_strAuthenticationFile);
			XmlDocument doc = new XmlDocument();
			doc.Load(strXmlFilePath);
			XmlElement root = doc.DocumentElement;
	    
			_strDefaultRole = root.Attributes["role"].Value;
	    
			_nodeUserInfo = root.SelectSingleNode(_strXQuery);
				    
			if (_nodeUserInfo == null)
				return false;       
	    
			if (_nodeUserInfo.SelectSingleNode("Password").InnerText == m_strHashedPassword)
				return true;        
			else        
				return false;
		}
	  
		private string DetermineRoles(string _strUserId)
		{			
			string strRoles = "";
			string strXQuery = "User[UserId = \"" + m_strUserId + "\"]";
	    
			for (int i = 0; i < m_arrRoles.Length; i++)
			{
				string strXmlFilePath = HttpContext.Current.Server.MapPath(m_arrRoles[i]);
	      
				XmlDocument doc = new XmlDocument();
				doc.Load(strXmlFilePath);
				XmlElement root = doc.DocumentElement;
	      
				string strRole = root.Attributes["role"].Value;          
	      
				XmlNode node = root.SelectSingleNode(strXQuery);
				if (node != null)
					strRoles += strRole + "|";
			}
	    
			return strRoles;
		}
	  
		private void CreateAuthenticationTicket(string _strRoles)
		{
			// Create an authentication ticket based on _strRoles, which expires in 20 minutes
			FormsAuthenticationTicket authTicket  = new FormsAuthenticationTicket(1, m_strUserId, DateTime.Now, DateTime.Now.AddMinutes(20), false, _strRoles);
	    
			// Encrypt the authentication ticket, store it in a string
			string strEncryptedTicket = FormsAuthentication.Encrypt(authTicket);
	    
			// Create an encrypted cookie from the encrypted ticket
			HttpCookie authCookie = new HttpCookie(FormsAuthentication.FormsCookieName, strEncryptedTicket);
	    
			// Add the cookie to the outgoing page
			HttpContext.Current.Response.Cookies.Add(authCookie);
		}
	  
		private void CreateUserInfoCookie(XmlNode _nodeUserInfo)
		{
			if (m_strCookieContent != null)
			{
				HttpCookie custCookie = new HttpCookie(m_strCookieName);
				string[] cookieContent = m_strCookieContent.Split(new char[] { ',' });
				foreach (string content in cookieContent)
				{
					string[] key_val = content.Split(new char[] { '=' });
					string key = key_val[0];
					string val = key_val[1];
					custCookie.Values.Add(key, _nodeUserInfo.SelectSingleNode(val).InnerText);
				}
				HttpContext.Current.Response.Cookies.Add(custCookie);
			}
		}
	} 
}