using System;                                     // EventArgs
using System.Web;                                 // HttpContext
using System.Web.UI;                              // LiteralControl
using System.Web.UI.WebControls;                  // Label, RadioButtonList, ListItem, etc...
using System.Xml;                                 // Xml Capabilities
using System.Web.Mail;                            // MailMessage & SmtpMail Classes
using System.ComponentModel;											// Needed for design-time support


[ assembly: TagPrefix("sstchur.web.survey", "sstchur") ]

namespace sstchur
{
  namespace web  
  {
		namespace survey
		{
			enum ButtonType { Back, Next, Submit };
			
			[ ToolboxData("<{0}:WebSurvey runat = \"server\"></{0}:WebSurvey>") ]
			[ Designer(typeof(sstchur.web.survey.designer.WebSurveyDesigner)) ]
			public class WebSurvey : System.Web.UI.WebControls.WebControl
			{	      
				// Private Member Variables
				private string m_strSurveyFile = "survey.xml";
				private string m_strAnswersFile = "answers.xml";
				private string m_strRedirectUrl = "";
				private string m_strCookieName = "WebSurvey:RepeatCookie";
				private string m_strEmailTo = "";
				private string m_strSmtpServer = "";
				private bool m_bAllowRepeats = true;
	      
				// Keep track of the various panels (groups) that web questions are separated into
				private System.Collections.ArrayList m_panels = new System.Collections.ArrayList();      
	            
				// Public Member Properties
				[
				Browsable(true),
				Category("Data"),
				DefaultValue("survey.xml"),
				Description("Relative path to the XML file containing the survey's question"),				
				]
				public string SurveyFile
				{
					get { return m_strSurveyFile; }
					set { m_strSurveyFile = value; }
				}
				
	      [
	      Browsable(true),
	      Category("Data"),
	      DefaultValue("answers.xml"),
	      Description("Relative path to the XML file that will store the survey's answers")
	      ]
				public string AnswersFile
				{
					get { return m_strAnswersFile; }
					set { m_strAnswersFile = value; }
				}
	      
	      [ BrowsableAttribute(false) ]
				public bool PreviouslyCompleted
				{
					get
					{
						if (CookieExists())
							return true;
						else
							return false;
					}
				}
	            
				// Constructor
				public WebSurvey()
				{
				}
	      
				protected override void OnInit(EventArgs e)
				{
					string strSurveyFile = HttpContext.Current.Server.MapPath(m_strSurveyFile);
					m_strAnswersFile = HttpContext.Current.Server.MapPath(m_strAnswersFile);
					
					XmlDocument doc = new XmlDocument();
					doc.Load(strSurveyFile);
					XmlElement root = doc.DocumentElement;
					
					// Setup the "allowrepeats" attribute
					XmlAttribute attrAllowRepeats = root.Attributes["allowrepeats"];
					if (attrAllowRepeats != null)
						m_bAllowRepeats = attrAllowRepeats.Value == "false" ? false : true;
					
					// Setup the "redirecturl" attribute
					XmlAttribute attrRedirectUrl = root.Attributes["redirecturl"];
					if (attrRedirectUrl != null)
						m_strRedirectUrl = attrRedirectUrl.Value;
						
					// Setup the "cookiename" attribute
					XmlAttribute attrCookieName = root.Attributes["cookiename"];
					if (attrCookieName != null)
						m_strCookieName = attrCookieName.Value;
						
					// Read the <EmailResults> tag (if it exists)
					XmlNode nodeEmailResults = root.SelectSingleNode("//WebSurvey/EmailResults");
					if (nodeEmailResults != null)
					{
						m_strEmailTo = nodeEmailResults.SelectSingleNode("EmailTo").InnerText;
						m_strSmtpServer = nodeEmailResults.SelectSingleNode("SmtpServer").InnerText;
					}
					    
					XmlNodeList nlGroups = root.SelectNodes("//WebSurvey/Group");
					CreateWebSurveyControls(nlGroups);        
					((Panel)m_panels[0]).Visible = true;
				}
	      
				private void CreateWebSurveyControls(XmlNodeList _groups)
				{
					// Each <Group> tag represents an <asp:Panel>, so we'll loop through all <Group>
					// tags in the XML file, and create an Panel object for each one
					foreach(XmlNode grp in _groups)
					{
						Panel p = new Panel();
						p.Visible = false;
						p.ID = grp.Attributes["id"].Value;
						m_panels.Add(p);
						
						// Get the "nextimage" attribute
						string strNextImage = "";
						XmlAttribute attrNextImage = grp.Attributes["nextimage"];
						if (attrNextImage != null)
							strNextImage = attrNextImage.Value;
							
						// Get the "backimage" attribute
						string strBackImage = "";
						XmlAttribute attrBackImage = grp.Attributes["backimage"];
						if (attrBackImage != null)
							strBackImage = attrBackImage.Value;
							
						// Select all child elements of the current <Group> tag.  This includes any and all tags, including
						// XML tags defined by this program, and any valid HTML tags						
						XmlNodeList children = grp.SelectNodes("*");
						foreach (XmlNode child in children)
							p.Controls.Add(CreateWebSurveyControl(child));
						
							
						// The panel's index (in the m_panels array) determines which buttons will be display at the bottom of it
						// If we're on the 1st panel - display only a [Next] button
						// If we're on the last panel - display a [Back] button and a [Finish] button
						// If we're on any panel inbetween - display both [Back] and [Next] buttons
						if (m_panels.IndexOf(p) > 0)
							AddButtonToPanel(p, ButtonType.Back, strBackImage);
							
						if (m_panels.IndexOf(p) == _groups.Count - 1)
							AddButtonToPanel(p, ButtonType.Submit, strNextImage);	
						else
							AddButtonToPanel(p, ButtonType.Next, strNextImage);
	          
						this.Controls.Add(p);
					}
				}
	      
				private void AddButtonToPanel(Panel _panel, ButtonType _buttonType, string _imgUrl)
				{	
					ImageButton btn = new ImageButton();
					btn.ImageUrl = _imgUrl;
		      
					switch (_buttonType)
					{
						case ButtonType.Back:
							btn.Command += new CommandEventHandler(btnBack_Click);
							btn.CommandName = _panel.ID;
							break;
			      	
						case ButtonType.Next:
							btn.Command += new CommandEventHandler(btnNext_Click);
							btn.CommandName = _panel.ID;
							break;
			      	
						case ButtonType.Submit:
							btn.Click += new ImageClickEventHandler(btnSubmit_Click);
							break;
					}
		      
					_panel.Controls.Add(btn);
					_panel.Controls.Add(new LiteralControl("&nbsp;&nbsp;&nbsp;"));					
				}
	      
				private int HidePanel(string _strPanelId)
				{
					Panel p = (Panel)this.FindControl(_strPanelId);
					p.Visible = false;
					int i = m_panels.IndexOf(p);        
					return i;
				}
	      
				private void btnBack_Click(object sender, CommandEventArgs e)
				{
					int i = HidePanel(e.CommandName);
					((Panel)m_panels[i-1]).Visible = true;      
				}
	      
				private void btnNext_Click(object sender, CommandEventArgs e)
				{
					if (Page.IsValid)
					{
						int i = HidePanel(e.CommandName);
						((Panel)m_panels[i+1]).Visible = true;
					}      
				}
	      
				private void btnSubmit_Click(Object sender, ImageClickEventArgs e)
				{
					if (Page.IsValid)
					{
						if (!System.IO.File.Exists(m_strAnswersFile))
						{
							XmlTextWriter writer = new XmlTextWriter(m_strAnswersFile, null);
							writer.Formatting = Formatting.Indented;
							writer.Indentation = 2;
	          
							writer.WriteStartDocument();
							writer.WriteStartElement("Answers");
							writer.WriteEndElement();
							writer.WriteEndDocument();
							writer.Flush();
							writer.Close();
						}
	          
						XmlDocument doc = new XmlDocument();
						doc.Load(m_strAnswersFile);
						XmlElement root = doc.DocumentElement;
	          
						XmlElement elemAnswerSet = doc.CreateElement("AnswerSet");
	  
						foreach (Control panel in this.Controls)
						{            
							Panel p = (Panel)panel;
							foreach (Control c in p.Controls)
							{              
								Type t = c.GetType().BaseType;
								if (t == typeof(sstchur.web.survey.WebQuestion))
								{
									WebQuestion wq = (WebQuestion)c;
									wq.SetAnswer();
									XmlElement elemAnswer = doc.CreateElement("Answer");
									XmlAttribute attribQuestionId = doc.CreateAttribute("questionId");
									attribQuestionId.Value = wq.Id;
									elemAnswer.Attributes.Append(attribQuestionId);
	    
									elemAnswer.InnerText = HttpContext.Current.Server.HtmlEncode(wq.Answer);
									elemAnswerSet.AppendChild(elemAnswer);          
								}
							}
						}
	          
						root.AppendChild(elemAnswerSet);
	          
						doc.Save(m_strAnswersFile);
	          
						if (!m_bAllowRepeats)
							CreateCookie();
	            
						if (m_strEmailTo != "")
							EmailAnswerSet(elemAnswerSet);
	          
						HttpContext.Current.Response.Redirect(m_strRedirectUrl);
					}  				
				}
				
				private void CreateCookie()
				{
					HttpCookie cookie = new HttpCookie(m_strCookieName);
					cookie.Values.Add("CompletedSurvey", "true");
					cookie.Expires = DateTime.Now.AddYears(5);
					HttpContext.Current.Response.Cookies.Add(cookie);
				}
				
				private bool CookieExists()
				{
					HttpCookie cookie = HttpContext.Current.Request.Cookies[m_strCookieName];
					if (cookie != null)
						return true;
					else
						return false;
				}			 			
	      
				private Control CreateWebSurveyControl(XmlNode _node)
				{
					string strTagName = _node.LocalName;
				
					if (strTagName == "Question")
					{					
						// Required Attributes
						string strId = _node.Attributes["id"].Value;
						string strType = _node.Attributes["type"].Value;
						string strQuestion = _node.SelectSingleNode("Statement").InnerXml;
						
						// Optional Attributes
						int nCols = 20;
						XmlAttribute attrCols = _node.Attributes["cols"];
						if (attrCols != null)
							nCols = Int32.Parse(attrCols.Value);
						
						int nRows = 4;
						XmlAttribute attrRows = _node.Attributes["rows"];
						if (attrRows != null)
							nRows = Int32.Parse(attrRows.Value);
						
						// Default layout for both MCSS and MCMS questions will be Vertical
						RepeatDirection rdLayout = RepeatDirection.Vertical;
						XmlAttribute attrLayout = _node.Attributes["layout"];
						if (attrLayout != null)
							rdLayout = attrLayout.Value == "horizontal" ? RepeatDirection.Horizontal : RepeatDirection.Vertical;
							
						string strBaseControl = "";
						XmlAttribute attrBaseControl = _node.Attributes["basecontrol"];
						if (attrBaseControl != null)
							strBaseControl = attrBaseControl.Value;
											
						bool bRequired = false;
						XmlAttribute attrRequired = _node.Attributes["required"];
						if (attrRequired != null)						
							bRequired = attrRequired.Value == "true" ? true : false;
						
						WebQuestion webQuestion;
						string[] arrResponses = null;					
						switch (strType)
						{
							case "shortans":
								webQuestion = new ShortAnsWebQuestion(strId, bRequired, strQuestion, nCols);
								break;
								
							case "essay":
								webQuestion = new EssayWebQuestion(strId, bRequired, strQuestion, nRows, nCols);
								break;
								
							case "mcss":								
								if (strBaseControl == "") strBaseControl = "radio";								
								arrResponses = GetResponses(_node);
								webQuestion = new McssWebQuestion(strId, bRequired, strQuestion, strBaseControl, rdLayout, arrResponses);
								break;
								
							case "mcms":
								if (strBaseControl == "") strBaseControl = "checkbox";								
								arrResponses = GetResponses(_node);
								webQuestion = new McmsWebQuestion(strId, bRequired, strQuestion, strBaseControl, rdLayout, arrResponses);
								break;
								
							default:
								webQuestion = new WebQuestion();
								break;
						}					
						return webQuestion;					
					}
					else if (strTagName == "Separator")
					{
						return new LiteralControl(_node.InnerXml);
					}
					else
					{
						return new LiteralControl("");
					}		
				}
	      
				private string[] GetResponses(XmlNode _node)
				{
					XmlNodeList nlResponses = _node.SelectNodes("Responses/Response");
					string[] arrResponses = new string[nlResponses.Count];

					for (int i = 0; i < nlResponses.Count; i++)
					{
						XmlNode n = nlResponses[i];
						arrResponses[i] = n.InnerText;
					}
					        
					return arrResponses;      
				}
	      
				protected override void Render(HtmlTextWriter writer)
				{
					Page.VerifyRenderingInServerForm(this);
					base.Render (writer);
				}
				
				private void EmailAnswerSet(XmlElement _elemAnswerSet)
				{ 
					XmlTextWriter writer = new XmlTextWriter(HttpContext.Current.Server.MapPath("~answerset.xml"), null);
					writer.Formatting = Formatting.Indented;
					writer.Indentation = 2;
	      
					writer.WriteStartDocument();
					writer.WriteStartElement("AnswerSet");        
					writer.WriteRaw(_elemAnswerSet.InnerXml);
					writer.WriteEndElement();
					writer.WriteEndDocument();
					writer.Flush();
					writer.Close();       
	        
					MailMessage mm = new MailMessage();
					mm.From = m_strEmailTo;
					mm.To = m_strEmailTo;
					mm.Subject = "Web Survey Answer Set (Web-generated Email)";
	        
					mm.BodyFormat = MailFormat.Html;					
					mm.Body = "WebSurvey Answer Set Attached";
	        
					mm.Attachments.Add(new MailAttachment(HttpContext.Current.Server.MapPath("~answerset.xml")));
	        
					SmtpMail.SmtpServer = m_strSmtpServer;
					SmtpMail.Send(mm);        
				}    
			}
    }
  }
} 